﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;
using Windows.System.Threading;
using System.Threading;


namespace Async_Metro
{
    partial class MainPage
    {
        const string pattern = @"<tr class=""rates_row"">\s*<td class=""currency"">(?<currency>[^<]+)</td>\s*<td class=""bidask"">(?<bid>[^<]+)</td>\s*<td class=""bidask"">(?<ask>[^<]+)</td>\s*<td class=""time"">(?<time>[^<]+)</td>\s*</tr>";

        public MainPage()
        {
            InitializeComponent();

            var ui = System.Threading.SynchronizationContext.Current;

            ThreadPoolTimer.CreatePeriodicTimer((timer) =>
            {
                ui.Post((a) =>
                {
                    this.Progress.Value = (this.Progress.Value >= 100) ? 0 : this.Progress.Value + 1;
                }, null);

            }, new TimeSpan(0, 0, 0, 0, 100));
        }

        private async void Button_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.Content.Text = "Waiting...";
                string base_url = @"http://www.oanda.com8/";
                string req_url = @"currency/real-time-rates";
                HttpClient client = new HttpClient();
                client.BaseAddress = new Uri(base_url);
                var response = await client.GetAsync(req_url);
                string content = response.Content.ReadAsString();
                string text = await ParseCurrencies(content);
                this.Content.Text = text;
            }
            catch (Exception ex)
            {
                this.Content.Text = ex.Message;
            }
        }
    
        private async Task<string> ParseCurrencies(string content)
        {
            // System.Threading.Thread.Sleep(5000);
            await Task.Delay(5000);
            Regex ex = new Regex(pattern);
            var matches = Regex.Matches(content, pattern, RegexOptions.IgnoreCase);
            StringBuilder builder = new StringBuilder();
            foreach (Match match in matches)
            {
                builder.AppendFormat(
                    "{0} BID={1} ASK={2} at {3}\n",
                    match.Groups["currency"],
                    match.Groups["bid"],
                    match.Groups["ask"],
                    match.Groups["time"]);
            }
            return builder.ToString();
        }

        private async Task<int> GetData1()
        {
            await Task.Delay(1000);
            return 1;
        }

        private async Task<int> GetData2()
        {
            await Task.Delay(1500);
            return 2;
        }

        private async Task<int> GetData3()
        {
            await Task.Delay(2000);
            return 3;
        }
        
        private async void SequentialAwait_Click(object sender, RoutedEventArgs e)
        {
            this.Content.Text = "Sequential running...";
            DateTime start = DateTime.Now;
            await GetData1();
            await GetData2();
            await GetData3();
            this.Content.Text = String.Format( "Sequential finished in {0} secs", (DateTime.Now - start).Seconds );
        }

        private async void TaskWaitAll_Click(object sender, RoutedEventArgs e)
        {
            this.Content.Text = "Task.WaitAll running...";
            DateTime start = DateTime.Now;
            var result = await Task.WhenAll(GetData1(), GetData2(), GetData3());
            this.Content.Text = String.Format("Sequential finished in {0} secs", (DateTime.Now - start).Seconds);
        }
        
        CancellationTokenSource cts = null;

        private async void Start_Click(object sender, RoutedEventArgs e)
        {
            this.Content.Text = "Start running...";
            try
            {
                DateTime start = DateTime.Now;
                cts = new CancellationTokenSource();
                // Use CancellationToken and a Progress<T> instance for Progress info (same as WinRT/.NET Calls)
                await DoSomeWork(
                    cts.Token,
                    new Progress<int>( (value) => 
                    {
                        this.ProgressCancel.Value = value;
                    }) );
                this.Content.Text = String.Format("Loop finished in {0} secs", (DateTime.Now - start).Seconds);
            }
            catch (OperationCanceledException ex)
            {
                this.Content.Text = String.Format("Operation canceled: {0}", ex.Message);
            }
        }

        private async Task DoSomeWork(CancellationToken cancel = default( CancellationToken ), IProgress<int> progress = default(IProgress<int>))
        {
            for (int i = 0; i < 100; i++)
            {
                if (progress != null)
                {
                    progress.Report(i);
                }
                await Task.Delay(100,cancel);
                if (cancel.IsCancellationRequested) return;
            }
        }

        private void Stop_Click(object sender, RoutedEventArgs e)
        {
            if (cts == null)
            {
                this.Content.Text = "No work running";
                return;
            }
            this.Content.Text = "Stopping...";
            cts.Cancel();
        }

    }
}
