﻿using System;
using System.Net;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;


namespace Async_WinForm
{
    public partial class Form1 : Form
    {
        const string pattern = @"<tr class=""rates_row"">\s*<td class=""currency"">(?<currency>[^<]+)</td>\s*<td class=""bidask"">(?<bid>[^<]+)</td>\s*<td class=""bidask"">(?<ask>[^<]+)</td>\s*<td class=""time"">(?<time>[^<]+)</td>\s*</tr>";

        public Form1()
        {
            InitializeComponent();

            Warmup();
        }

        // 01 - sequential button1
        private async void button1_Click(object sender, EventArgs e)
        {
            richTextBox1.Clear();
            string url = @"http://www.oanda.com/currency/real-time-rates";
            WebRequest req = WebRequest.Create(url);
            WebResponse response = await req.GetResponseAsync();
            StreamReader reader = new StreamReader( response.GetResponseStream() );
            string content = await reader.ReadToEndAsync();
            string text = await ParseCurrencies(content);
            richTextBox1.Text = text;
        }
 


        // 01 - sequential ParseCurrencies
        private async Task<string> ParseCurrencies(string content)
        {
            // System.Threading.Thread.Sleep(5000);
            await Task.Delay(5000);
            Regex ex = new Regex(pattern);
            var matches = Regex.Matches(content, pattern, RegexOptions.IgnoreCase);
            StringBuilder builder = new StringBuilder();
            foreach (Match match in matches)
            {
                builder.AppendFormat(
                    "{0} BID={1} ASK={2} at {3}\n",
                    match.Groups["currency"],
                    match.Groups["bid"],
                    match.Groups["ask"],
                    match.Groups["time"]);
            }
            return builder.ToString();
        }


        private void timer1_Tick(object sender, EventArgs e)
        {
            progressBar1.Value = (progressBar1.Value >= 100) ? 0 : progressBar1.Value + 1;
        }

        // Async warmup - load .NET classes for the first time
        private async void Warmup()
        {
            string url = @"http://www.bing.com";
            WebRequest req = WebRequest.Create(url);
            WebResponse response = await req.GetResponseAsync();
            StreamReader reader = new StreamReader(response.GetResponseStream());
            string content = await reader.ReadToEndAsync();

        }
    }


    
}



/*

        // 01 - sequential button1
        private void button1_Click(object sender, EventArgs e)
        {
            richTextBox1.Clear();
            string url = @"http://www.oanda.com/currency/real-time-rates";
            WebRequest req = WebRequest.Create(url);
            WebResponse response = req.GetResponse();
            StreamReader reader = new StreamReader( response.GetResponseStream() );
            string content = reader.ReadToEnd();
            string text = ParseCurrencies(content);
            richTextBox1.Text = text;
        }

        // 01 - sequential ParseCurrencies
        private string ParseCurrencies(string content)
        {
            System.Threading.Thread.Sleep(5000);
            // await Task.Delay(5000);
            Regex ex = new Regex(pattern);
            var matches = Regex.Matches(content, pattern, RegexOptions.IgnoreCase);
            StringBuilder builder = new StringBuilder();
            foreach (Match match in matches)
            {
                builder.AppendFormat(
                    "{0} BID={1} ASK={2} at {3}\n",
                    match.Groups["currency"],
                    match.Groups["bid"],
                    match.Groups["ask"],
                    match.Groups["time"]);
            }
            return builder.ToString();
        }

 
        // 02 - Task button1
        private void button1_Click(object sender, EventArgs e)
        {
            var contextUserInterface = TaskScheduler.FromCurrentSynchronizationContext();
            richTextBox1.Clear();
            string url = @"http://www.oanda.com/currency/real-time-rates";
            WebRequest req = WebRequest.Create(url);
            req.GetResponseAsync().ContinueWith(
                (response) =>
                {
                    StreamReader reader = new StreamReader(response.Result.GetResponseStream());
                    reader.ReadToEndAsync().ContinueWith(
                        (content) =>
                        {
                            string text = ParseCurrencies(content.Result);
                            return text;
                        }).ContinueWith((text) =>
                        {
                            richTextBox1.Text = text.Result;
                        }, contextUserInterface ); ;
                });
        }

        // 02 sequential ParseCurrencies
        private string ParseCurrencies(string content)
        {
            System.Threading.Thread.Sleep(5000);
            // await Task.Delay(5000);
            Regex ex = new Regex(pattern);
            var matches = Regex.Matches(content, pattern, RegexOptions.IgnoreCase);
            StringBuilder builder = new StringBuilder();
            foreach (Match match in matches)
            {
                builder.AppendFormat(
                    "{0} BID={1} ASK={2} at {3}\n",
                    match.Groups["currency"],
                    match.Groups["bid"],
                    match.Groups["ask"],
                    match.Groups["time"]);
            }
            return builder.ToString();
        }
 
 
 
        // 03 - async button1
        private async void button1_Click(object sender, EventArgs e)
        {
            richTextBox1.Clear();
            string url = @"http://www.oanda.com/currency/real-time-rates";
            WebRequest req = WebRequest.Create(url);
            WebResponse response = await req.GetResponseAsync();
            StreamReader reader = new StreamReader( response.GetResponseStream() );
            string content = await reader.ReadToEndAsync();
            string text = await ParseCurrencies(content);
            richTextBox1.Text = text;
        }
 
        // 03 async ParseCurrencies
        private async Task<string> ParseCurrencies(string content)
        {
            //System.Threading.Thread.Sleep(5000);
            await Task.Delay(5000);
            Regex ex = new Regex(pattern);
            var matches = Regex.Matches(content, pattern, RegexOptions.IgnoreCase);
            StringBuilder builder = new StringBuilder();
            foreach (Match match in matches)
            {
                builder.AppendFormat(
                    "{0} BID={1} ASK={2} at {3}\n",
                    match.Groups["currency"],
                    match.Groups["bid"],
                    match.Groups["ask"],
                    match.Groups["time"]);
            }
            return builder.ToString();
        }

*/